#import "ViewController.h"
#include <stdio.h>


namespace 
{
	NSString* Filename = @"test.bin";
	NSString* KVPKey = @"testkey";
	
	NSString* CreateFullFilenameInAppDir()
	{
		NSArray* paths = NSSearchPathForDirectoriesInDomains(NSApplicationSupportDirectory, NSUserDomainMask, YES); 
		NSString* appSupportDir = [paths objectAtIndex:0];
		NSString* executableName = [[[NSBundle mainBundle] infoDictionary] objectForKey:@"CFBundleExecutable"];
		NSString* appDir = [appSupportDir stringByAppendingPathComponent:executableName];
		[[NSFileManager defaultManager] createDirectoryAtPath:appDir withIntermediateDirectories:YES attributes:nil error:nil];
		NSString* fullFilename = [[appDir stringByAppendingPathComponent:Filename] retain];
		return fullFilename;
	}
}


@implementation ViewController

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Release any cached data, images, etc that aren't in use.
}

#pragma mark - View lifecycle

- (void)viewDidLoad
{
    [super viewDidLoad];

	Class gcClass = (NSClassFromString(@"UIDocument"));
	if (gcClass != NULL)
		m_iCloudDir = [[[NSFileManager defaultManager] URLForUbiquityContainerIdentifier:nil] retain];


	if (m_iCloudDir != NULL)
	{
		NSLog(@"iCloud access at %@", m_iCloudDir);
		m_fullFilename = [[[m_iCloudDir path] stringByAppendingPathComponent:Filename] retain];
	}
	else
	{
		NSLog(@"No iCloud.");
		m_fullFilename = CreateFullFilenameInAppDir();
	}
}

- (void)viewDidUnload
{
    [super viewDidUnload];
	[m_status release];
	[m_text release];
	[m_textField release];
	
	[m_iCloudDir release];
	[m_fullFilename release];
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
}



- (void)loadData:(NSString*)filename
{
	char buffer[1024];

	FILE* fin = fopen([filename UTF8String], "rb");
	fseek (fin , 0 , SEEK_END);
	const int size = ftell (fin);
	rewind (fin);
  	assert(size < 1024);
	
	fread(buffer, 1, size, fin);
	
	fclose(fin);
	
	m_text.text = [NSString stringWithUTF8String:buffer];
}


- (void)createFile:(NSString*)filename
{
	const char* data = "Default file contents.";
	FILE* fout = fopen([filename UTF8String], "wb");
	assert(fout != NULL);
	fwrite(data, strlen(data)+1, 1, fout);
	fclose(fout);
}


- (void)addFileToiCloud
{
	NSError* error;
	NSString* srcFilename = CreateFullFilenameInAppDir();
	NSURL* srcURL = [NSURL fileURLWithPath:srcFilename];
	NSURL* dstFilename = [m_iCloudDir URLByAppendingPathComponent:Filename];
	bool ok = [[NSFileManager defaultManager] setUbiquitous:YES itemAtURL:srcURL destinationURL:dstFilename error:&error];
	[srcFilename release];
	if (!ok)
	{
		NSLog(@"Could not move file");
	}
}


- (void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
}


- (void)viewWillDisappear:(BOOL)animated
{
	[super viewWillDisappear:animated];
}

- (void)viewDidDisappear:(BOOL)animated
{
	[super viewDidDisappear:animated];
}


- (void)updateKVPFromiCloud
{
	if (m_iCloudDir == NULL)
		return;

	NSUbiquitousKeyValueStore* store = [NSUbiquitousKeyValueStore defaultStore];
	NSString* data = [store stringForKey:KVPKey];

	NSUserDefaults* defaults = [NSUserDefaults standardUserDefaults];
	[defaults setValue:data forKey:KVPKey];
	[defaults synchronize];
}


- (void)updateiCloudFromKVP
{
	if (m_iCloudDir == NULL)
		return;

	NSUserDefaults* defaults = [NSUserDefaults standardUserDefaults];
	NSString* data = [defaults stringForKey:KVPKey];
	
	NSUbiquitousKeyValueStore* store = [NSUbiquitousKeyValueStore defaultStore];
	[store setString:data forKey:KVPKey];
	[store synchronize];
}


- (void) refreshData
{
	[self view];

	NSFileManager* fileMgr = [NSFileManager defaultManager];
	if (m_iCloudDir == NULL)
	{
		m_status.text = @"No iCloud";
		if (![fileMgr fileExistsAtPath:m_fullFilename])
			[self createFile:m_fullFilename];
	}
	else
	{
		m_status.text = @"Checking";

		// Check if file already in iCloud
		NSURL* fileURL = [m_iCloudDir URLByAppendingPathComponent:Filename];
		NSNumber* isIniCloud = nil;
		if (![fileURL getResourceValue:&isIniCloud forKey:NSURLIsUbiquitousItemKey error:nil] || ![isIniCloud boolValue])
		{
			m_status.text = @"Created iCloud file first time";
			NSString* fullFilename = CreateFullFilenameInAppDir();
			[self createFile:fullFilename];
			[self loadData:fullFilename];
			[fullFilename release];
			[self performSelectorInBackground:@selector(addFileToiCloud) withObject:NULL];
			return;
		}
		
		NSNumber* isDownloaded = nil;
		if (![fileURL getResourceValue:&isDownloaded forKey:NSURLUbiquitousItemIsDownloadedKey error:nil]) 
		{
			NSLog(@"Error");
			assert(0);
			return;
		}		
		if (![isDownloaded boolValue])
		{
			m_status.text = @"Downloading";
			[fileMgr startDownloadingUbiquitousItemAtURL:fileURL error:nil];

			m_timer = [NSTimer timerWithTimeInterval:0.5f target:self selector:@selector(checkDownloadFileStatus) userInfo:nil repeats:YES];
			[[NSRunLoop currentRunLoop] addTimer:m_timer forMode:NSDefaultRunLoopMode];			
			return;
		}
		
		m_status.text = @"Ready (at start)";
		
		[self updateKVPFromiCloud];
	}


	NSUserDefaults* defaults = [NSUserDefaults standardUserDefaults];
	NSString* data = [defaults stringForKey:KVPKey];
	m_textField.text = data;

	[self loadData:m_fullFilename];
}


- (void)checkDownloadFileStatus
{
	NSFileManager* fileMgr = [NSFileManager defaultManager];
	NSURL* ubiq = [fileMgr URLForUbiquityContainerIdentifier:nil];
	NSString* filename = @"test.bin";
	NSURL* fileURL = [ubiq URLByAppendingPathComponent:filename];
	NSNumber* isDownloaded = nil;
	if (![fileURL getResourceValue:&isDownloaded forKey:NSURLUbiquitousItemIsDownloadedKey error:nil]) 
		return;
		
	if (![isDownloaded boolValue])
		return;
		

	[m_timer invalidate];
	m_timer = NULL;
	
	m_status.text = @"Ready (after download)";
	[self loadData:m_fullFilename];
}


- (IBAction) save
{
	const char* text = [m_text.text UTF8String];
	
	FILE* fout = fopen([m_fullFilename UTF8String], "wb");
	fwrite(text, strlen(text)+1, 1, fout);
	fclose(fout);
}


- (void)textFieldDidEndEditing:(UITextField *)textField
{
	NSString* data = textField.text;

	NSUserDefaults* defaults = [NSUserDefaults standardUserDefaults];
	[defaults setValue:data forKey:KVPKey];
	[defaults synchronize];
	
	[self updateiCloudFromKVP];
}


@end
